"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
// Durations defined by ISO 31-1
var MILLISECONDS_IN_A_SECOND = 1000;
var MILLISECONDS_IN_A_MINUTE = MILLISECONDS_IN_A_SECOND * 60;
var MILLISECONDS_IN_AN_HOUR = MILLISECONDS_IN_A_MINUTE * 60;
var MILLISECONDS_IN_A_DAY = MILLISECONDS_IN_AN_HOUR * 24;
// Durations defined by common sense
var MILLISECONDS_IN_A_WEEK = MILLISECONDS_IN_A_DAY * 7;
var MILLISECONDS_IN_A_YEAR = MILLISECONDS_IN_A_DAY * 365;
var MILLISECONDS_IN_A_MONTH = MILLISECONDS_IN_A_YEAR / 12;
exports.ZERO = Object.freeze({
    years: 0,
    months: 0,
    weeks: 0,
    days: 0,
    hours: 0,
    minutes: 0,
    seconds: 0,
    milliseconds: 0,
});
// TODO: ISOCharacter and ISOPrecision no longer needed to parse duration string. Can they be removed? Needed still to stringify?
exports.UNITS_MAP = {
    years: {
        milliseconds: MILLISECONDS_IN_A_YEAR,
        // TODO: Test adding a year on leap year
        addToDate: function (date, value) { return date.setFullYear(date.getFullYear() + value); },
        dateGetter: function (date) { return date.getFullYear(); },
        ISOCharacter: 'Y',
        ISOPrecision: 'period',
    },
    months: {
        milliseconds: MILLISECONDS_IN_A_MONTH,
        addToDate: function (date, value) {
            // TODO: Test me:
            // https://stackoverflow.com/questions/2706125/javascript-function-to-add-x-months-to-a-date
            var day = date.getDate();
            date.setMonth(date.getMonth() + value, 1);
            var month = date.getMonth();
            date.setDate(day);
            if (date.getMonth() !== month) {
                date.setDate(0);
            }
            return date.getMonth();
        },
        dateGetter: function (date) { return date.getMonth(); },
        ISOCharacter: 'M',
        ISOPrecision: 'period',
    },
    weeks: {
        milliseconds: MILLISECONDS_IN_A_WEEK,
        addToDate: function (date, value) { return date.setDate(date.getDate() + (value * 7)); },
        dateGetter: function () { return 0; },
        ISOCharacter: 'W',
        ISOPrecision: 'period',
        stringifyConvertTo: 'days',
    },
    days: {
        milliseconds: MILLISECONDS_IN_A_DAY,
        addToDate: function (date, value) { return date.setDate(date.getDate() + value); },
        dateGetter: function (date) { return date.getDate(); },
        ISOCharacter: 'D',
        ISOPrecision: 'period',
    },
    hours: {
        milliseconds: MILLISECONDS_IN_AN_HOUR,
        addToDate: function (date, value) { return date.setHours(date.getHours() + value); },
        dateGetter: function (date) { return date.getHours(); },
        ISOCharacter: 'H',
        ISOPrecision: 'time',
    },
    minutes: {
        milliseconds: MILLISECONDS_IN_A_MINUTE,
        addToDate: function (date, value) { return date.setMinutes(date.getMinutes() + value); },
        dateGetter: function (date) { return date.getMinutes(); },
        ISOCharacter: 'M',
        ISOPrecision: 'time',
    },
    seconds: {
        milliseconds: MILLISECONDS_IN_A_SECOND,
        addToDate: function (date, value) { return date.setSeconds(date.getSeconds() + value); },
        dateGetter: function (date) { return date.getSeconds(); },
        ISOCharacter: 'S',
        ISOPrecision: 'time',
    },
    milliseconds: {
        milliseconds: 1,
        addToDate: function (date, value) { return date.setMilliseconds(date.getMilliseconds() + value); },
        dateGetter: function (date) { return date.getMilliseconds(); },
        stringifyConvertTo: 'seconds',
    },
};
exports.UNIT_KEYS = [
    'years',
    'months',
    'weeks',
    'days',
    'hours',
    'minutes',
    'seconds',
    'milliseconds',
];
exports.UNITS = exports.UNIT_KEYS.map(function (unit) { return (__assign({}, exports.UNITS_MAP[unit], { unit: unit })); });
//# sourceMappingURL=units.js.map