"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var units_1 = require("./lib/units");
var numberUtils_1 = require("./lib/numberUtils");
var between_1 = require("./between");
var apply_1 = require("./apply");
var toUnit_1 = require("./toUnit");
var subtract_1 = require("./subtract");
var parse_1 = require("./parse");
// TODO: Rename "normalizeApprox" since it should no longer be lossy
var normalizeApprox = function (duration) {
    var _a = parse_1.parse(duration), years = _a.years, months = _a.months, weeks = _a.weeks, days = _a.days, rest = __rest(_a, ["years", "months", "weeks", "days"]);
    var output = __assign({}, rest, { years: years + numberUtils_1.floorTowardsZero(months / 12), months: months % 12 || 0, weeks: 0, days: 0 });
    var remaining = __assign({}, rest, { years: 0, months: 0, weeks: 0, days: days + (weeks * 7) });
    [
        'days',
        'hours',
        'minutes',
        'seconds',
        'milliseconds',
    ].forEach(function (unit) {
        var _a;
        var milliseconds = units_1.UNITS_MAP[unit].milliseconds;
        output[unit] = numberUtils_1.floorTowardsZero(toUnit_1.toMilliseconds(remaining) / milliseconds);
        remaining = subtract_1.subtract(remaining, (_a = {}, _a[unit] = output[unit], _a));
    });
    return output;
};
var normalizeRelative = function (duration, referenceDate) { return between_1.between(referenceDate, apply_1.apply(referenceDate, duration)); };
/**
 * Convert a `Time` object or number of milliseconds into a complete
 * `Time` object that expresses the duration in the most appropriate units.
 *
 * @example
 * normalize({ milliseconds 4000 })
 * // { remaining.years: 0, months: 0, weeks: 0, days: 0, hours: 0, minutes: 0, seconds: 4, milliseconds: 0 }
 */
exports.normalize = function (duration, referenceDate) {
    if (referenceDate !== undefined) {
        return normalizeRelative(duration, referenceDate);
    }
    return normalizeApprox(duration);
};
//# sourceMappingURL=normalize.js.map