"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var parse_1 = require("./parse");
var isZero_1 = require("./isZero");
var getUnitCount_1 = require("./lib/getUnitCount");
var units_1 = require("./lib/units");
var joinComponents = function (component) {
    return component
        .join('')
        // Commas are mentioned in the spec as the preferred decimal delimiter
        .replace(/\./g, ',');
};
exports.toString = function (duration) {
    // Zero values are a special case, since "P" is not a valid value.
    // At least one unit must be specified.
    if (isZero_1.isZero(duration)) {
        return 'P0D';
    }
    var parsedTime = __assign({}, parse_1.parse(duration));
    // Weeks should not be included in the output, unless it is the only unit.
    if (getUnitCount_1.getUnitCount(parsedTime) === 1 && parsedTime.weeks !== 0) {
        return "P" + parsedTime.weeks + "W";
    }
    var components = {
        period: [],
        time: [],
    };
    // Some units should be converted before stringifying.
    // For example, weeks should not be mixed with other units, and milliseconds
    // don't exist on ISO duration strings.
    units_1.UNITS.forEach(function (_a) {
        var fromUnit = _a.unit, toUnit = _a.stringifyConvertTo;
        if (toUnit === undefined) {
            return;
        }
        var millisecondValue = parsedTime[fromUnit] * units_1.UNITS_MAP[fromUnit].milliseconds;
        parsedTime[toUnit] += millisecondValue / units_1.UNITS_MAP[toUnit].milliseconds;
        parsedTime[fromUnit] = 0;
    });
    // Push each non-zero unit to its relevant array
    units_1.UNITS.forEach(function (_a) {
        var unit = _a.unit, ISOPrecision = _a.ISOPrecision, ISOCharacter = _a.ISOCharacter;
        var value = parsedTime[unit];
        if (ISOPrecision !== undefined && value !== 0) {
            components[ISOPrecision].push("" + value + ISOCharacter);
        }
    });
    // Build output string
    var output = "P" + joinComponents(components.period);
    if (components.time.length) {
        output += "T" + joinComponents(components.time);
    }
    return output;
};
//# sourceMappingURL=toString.js.map